# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['chrome_local_storage']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.25.0,<3.0.0', 'trio-chrome-devtools-protocol>=0.6.0,<0.7.0']

setup_kwargs = {
    'name': 'chrome-local-storage',
    'version': '0.5.1',
    'description': 'Interact with Google Chrome local storage',
    'long_description': '# Chrome Local Storage\n\nThis Python package makes it easy to interact with Google Chrome local storage,\neither a locally-running browser or any remote browser that supports remote\ndebugging (e.g. Chrome on Android via `adb` port forwarding).\n\n\n## Prerequisites\n\nInstallation is via `pip`:\n\n```bash\npip install chrome-local-storage\n```\n\nChrome must be running with the debugging port active for the library\nto connect. There are various methods to do this, for example, on Windows:\n\n```\nchrome.exe --remote-debugging-port=9222\n```\n\nAnd on MacOS:\n\n```\n/Applications/Google\\ Chrome.app/Contents/MacOS/Google\\ Chrome --remote-debugging-port=9222\n```\n\nThe library will also work on any mobile device that supports remote debugging on Chrome,\nFor example, on Android, use the [Android Debug Bridge](https://developer.android.com/studio/command-line/adb)\nto set up a port forward as follows:\n\n```bash\nadb forward tcp:9222 localabstract:chrome_devtools_remote\n```\n\nIn all of the above examples, the debugger will be available at `localhost:9222`\nwhich is what the library expects by default.\n\n\n## Usage\n\nBasic usage is as follows:\n\n```python3\nimport chrome_local_storage\n\nlocal_storage = chrome_local_storage.ChromeLocalStorage()\n\nlocal_storage.set(\'example.com\', \'my-key\', \'my-value\')\nvalue = local_storage.get(\'example.com\', \'my-key\')\nprint(value)\n```\n\nThe first parameter in both `get` and `set` determines the page\nwhose local storage will be used for the operation. The page must\nalready be open in the browser, and it does not have to be an exact\nmatch to the whole URL as long as it\'s unique across open pages.\n\nIn a more complex example, two constructors connect to two different\nChrome instances and copy Wordle statistics from one to the other\n(the desire to transfer my streak from one device to another was\nthe original motivation for building this library).\n\n```bash\nchrome --remote-debugging-port=9222 "https://nytimes.com/games/wordle"\nadb forward tcp:9223 localabstract:chrome_devtools_remote\n```\n\n```python3\nimport chrome_local_storage\n\nlaptop_storage = chrome_local_storage.ChromeLocalStorage(port=9222)\nphone_storage = chrome_local_storage.ChromeLocalStorage(port=9223)\n\nwordle_stats = laptop_storage.get(\'games/wordle\', \'nyt-wordle-statistics\')\nphone_storage.set(\'games/wordle\', \'nyt-wordle-statistics\', wordle_stats)\n```\n',
    'author': 'Judson Neer',
    'author_email': 'judson.neer@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/lordjabez/chrome-local-storage',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<3.10',
}


setup(**setup_kwargs)



__version__ = '0.5.1'


import requests
import trio
import trio_cdp


class BrowserConnectionError(Exception):
    """Exception thrown when browser connectivity cannot be established."""
    def __init__(self, host):
        message = f'Could not find tabs on {host} with debugging URLs. Is Chrome running with debugging enabled?'
        super().__init__(message)


class PageNotFoundError(Exception):
    """Exception thrown when no matching URL is found."""
    def __init__(self, url=None):
        message = f'Could not match an open URL to {url}. Is the page open in Chrome?'
        super().__init__(message)


class KeyNotFoundError(Exception):
    """Exception thrown when the key is not found in local storage."""
    def __init__(self, key):
        message = f'Could not find key {key} in local storage.'
        super().__init__(message)


class KeyNotSetError(Exception):
    """Exception thrown when setting the key in local storage fails."""
    def __init__(self, key):
        message = f'Could not set key {key} in local storage.'
        super().__init__(message)


class ChromeLocalStorage():
    """Interact with Google Chrome local storage."""

    def _get_debugger_url(self, host):
        try:
            tabs = requests.get(f'http://{host}/json').json()
            return next(t['webSocketDebuggerUrl'] for t in tabs if 'webSocketDebuggerUrl' in t)
        except Exception:
            raise BrowserConnectionError()

    async def _get_ids(self, connection, target):
        async with connection.open_session(target.target_id):
            resource_tree = await trio_cdp.page.get_resource_tree()
            security_origin = resource_tree.frame.security_origin
            storage_id = trio_cdp.dom_storage.StorageId(security_origin, True)
        return (target.target_id, storage_id)

    async def _build_id_map(self, host, port):
        async with trio_cdp.open_cdp(self._debugger_url) as connection:
            targets = await trio_cdp.target.get_targets()
            targets_with_url = (t for t in targets if hasattr(t, 'url'))
            self._id_map = [(t.url, await self._get_ids(connection, t)) for t in targets_with_url]

    async def _get_ids_for_url(self, url):
        try:
            return next(i for u, i in self._id_map if url in u)
        except Exception:
            raise PageNotFoundError(url)

    async def _get_storage_item(self, url, key):
        target_id, storage_id = await self._get_ids_for_url(url)
        async with trio_cdp.open_cdp(self._debugger_url) as connection:
            async with connection.open_session(target_id):
                try:
                    storage_items = await trio_cdp.dom_storage.get_dom_storage_items(storage_id)
                    return next(i[1] for i in storage_items if i[0] == key)
                except Exception:
                    raise KeyNotFoundError(key)

    async def _set_storage_item(self, url, key, value):
        target_id, storage_id = await self._get_ids_for_url(url)
        async with trio_cdp.open_cdp(self._debugger_url) as connection:
            async with connection.open_session(target_id):
                try:
                    await trio_cdp.dom_storage.set_dom_storage_item(storage_id, key, value)
                except Exception:
                    raise KeyNotSetError(key)

    def get(self, url, key):
        """
        Get an item from local storage.

        :param url: Get item from page whose URL contains this value
        :param key: Key of local storage to get
        :return: Value of local storage item
        """
        return trio.run(self._get_storage_item, url, key)

    def set(self, url, key, value):
        """
        Set an item in local storage.

        :param url: Set item on page whose URL contains this value
        :param key: Key of local storage to set
        :param value: Value to put in local storage
        """
        trio.run(self._set_storage_item, url, key, value)

    def __init__(self, host='localhost', port=9222):
        """
        Construct an instance of the class.

        :param host: hostname running the Chrome debugger, defaults to localhost
        :param port: port running the Chrome debugger, defaults to 9222
        """
        self._debugger_url = self._get_debugger_url(f'{host}:{port}')
        trio.run(self._build_id_map, host, port)
